﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace TreeViewTest
{
    using System.Collections.Specialized;
    using System.Windows;
    using System.Windows.Controls;
    using System.Windows.Data;

    using Infragistics.Windows.DataPresenter;
    using Infragistics.Windows.DataPresenter.Events;

    public class ExtendedGrid : XamDataGrid
    {
        protected override void OnFieldLayoutInitialized(FieldLayoutInitializedEventArgs args)
        {
            base.OnFieldLayoutInitialized(args);

            this.SetupColumnMenus();

            foreach (FieldLayout colLayout in this.FieldLayouts)
            {
                colLayout.Fields.CollectionChanged += this.OnFieldsChanged;
            }
        }

        private void OnFieldsChanged(object sender, NotifyCollectionChangedEventArgs e)
        {
            this.SetupColumnMenus();
        }

        private void SetupColumnMenus()
        {
            Style originalStyle = this.FindResource(typeof(LabelPresenter)) as Style;

            foreach (FieldLayout colLayout in this.FieldLayouts)
            {
                ContextMenu columnSubMenu = new ContextMenu();

                Style headerPresenterStyle = new Style(typeof(DataRecordPresenter));
                Trigger isHeaderRecordTrigger = new Trigger { Property = DataRecordPresenter.IsHeaderRecordProperty, Value = true };
                headerPresenterStyle.Triggers.Add(isHeaderRecordTrigger);
                isHeaderRecordTrigger.Setters.Add(new Setter(ContextMenuProperty, columnSubMenu));
                colLayout.Settings.DataRecordPresenterStyle = headerPresenterStyle;

                Style newSubStyle = new Style(typeof(LabelPresenter));
                newSubStyle.BasedOn = originalStyle;
                newSubStyle.Setters.Add(new Setter(ContextMenuProperty, columnSubMenu));

                foreach (Field col in colLayout.Fields.OrderBy(x => x.Label))
                {
                    SetupColumnContextMenu(newSubStyle, columnSubMenu, col);
                }
            }
        }

        private static void SetupColumnContextMenu(Style styleWithContextMenu, ItemsControl columnMenu, Field col)
        {
            if (col == null || col.Settings == null)
            {
                return;
            }

            if (!col.HasLabel)
            {
                return;
            }

            col.Settings.LabelPresenterStyle = styleWithContextMenu;

            MenuItem item = new MenuItem() { Header = col.Label };
            item.IsChecked = col.Visibility == Visibility.Visible;
            BindingOperations.SetBinding(col, Field.VisibilityProperty, new Binding() { Source = item, Path = new PropertyPath(MenuItem.IsCheckedProperty), Converter = new BooleanToVisibilityConverter(), Mode = BindingMode.OneWay });
            item.IsCheckable = true;
            columnMenu.Items.Add(item);
        }
    }
}
